import math
import struct

import lzf
from formant.protos.model.v1.media_pb2 import PointCloud
import numpy as np


def laserscan_to_formant_pointcloud(message):
    # type (...) -> PointCloud
    height = 1
    width = len(message.ranges)
    count = height * width

    xs = np.zeros((count,), dtype="float32")
    ys = np.zeros((count,), dtype="float32")
    zs = np.zeros((count,), dtype="float32")

    for i in range(len(message.ranges)):
        xf = 0.0
        yf = 0.0
        zf = 0.0

        r = message.ranges[i]
        a = message.angle_min + message.angle_increment * i
        if a > message.angle_max:
            break
        if r > message.range_min and r < message.range_max:
            xf = r * math.cos(a)
            yf = r * math.sin(a)

        xs[i] = xf
        ys[i] = yf
        zs[i] = zf

    points = np.concatenate((xs, ys, zs)).tobytes()

    buffer = (
        """VERSION 0.7
FIELDS x y z rgb
SIZE 4 4 4 4
TYPE F F F F
WIDTH %s
HEIGHT 1
DATA binary_compressed
"""
        % count
    )
    compressed = lzf.compress(points, 2 * len(points))
    raw = (
        buffer.encode("utf-8")
        + struct.pack("<I", len(compressed))
        + struct.pack("<I", len(points))
        + compressed
    )

    return raw



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/model/v1/math.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/model/v1/math.proto',
  package='v1.model',
  syntax='proto3',
  serialized_options=b'Z)github.com/FormantIO/genproto/go/v1/model',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1aprotos/model/v1/math.proto\x12\x08v1.model\"\x1f\n\x07Numeric\x12\x14\n\x05value\x18\x01 \x01(\x01R\x05value\"Q\n\x0fNumericSetEntry\x12\x14\n\x05value\x18\x01 \x01(\x01R\x05value\x12\x14\n\x05label\x18\x02 \x01(\tR\x05label\x12\x12\n\x04unit\x18\x03 \x01(\tR\x04unit\"C\n\nNumericSet\x12\x35\n\x08numerics\x18\x01 \x03(\x0b\x32\x19.v1.model.NumericSetEntryR\x08numerics\"-\n\x03\x42it\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\x08R\x05value\"+\n\x06\x42itset\x12!\n\x04\x62its\x18\x01 \x03(\x0b\x32\r.v1.model.BitR\x04\x62its\"_\n\x05Twist\x12)\n\x06linear\x18\x01 \x01(\x0b\x32\x11.v1.model.Vector3R\x06linear\x12+\n\x07\x61ngular\x18\x02 \x01(\x0b\x32\x11.v1.model.Vector3R\x07\x61ngular\"r\n\tTransform\x12\x33\n\x0btranslation\x18\x01 \x01(\x0b\x32\x11.v1.model.Vector3R\x0btranslation\x12\x30\n\x08rotation\x18\x02 \x01(\x0b\x32\x14.v1.model.QuaternionR\x08rotation\"\x87\x01\n\x0eTransformFrame\x12!\n\x0cparent_frame\x18\x01 \x01(\tR\x0bparentFrame\x12\x1f\n\x0b\x63hild_frame\x18\x02 \x01(\tR\nchildFrame\x12\x31\n\ttransform\x18\x03 \x01(\x0b\x32\x13.v1.model.TransformR\ttransform\"3\n\x07Vector3\x12\x0c\n\x01x\x18\x01 \x01(\x01R\x01x\x12\x0c\n\x01y\x18\x02 \x01(\x01R\x01y\x12\x0c\n\x01z\x18\x03 \x01(\x01R\x01z\"1\n\x05Point\x12\x0c\n\x01x\x18\x01 \x01(\x01R\x01x\x12\x0c\n\x01y\x18\x02 \x01(\x01R\x01y\x12\x0c\n\x01z\x18\x03 \x01(\x01R\x01z\"D\n\nQuaternion\x12\x0c\n\x01x\x18\x01 \x01(\x01R\x01x\x12\x0c\n\x01y\x18\x02 \x01(\x01R\x01y\x12\x0c\n\x01z\x18\x03 \x01(\x01R\x01z\x12\x0c\n\x01w\x18\x04 \x01(\x01R\x01wB+Z)github.com/FormantIO/genproto/go/v1/modelb\x06proto3'
)




_NUMERIC = _descriptor.Descriptor(
  name='Numeric',
  full_name='v1.model.Numeric',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.Numeric.value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=71,
)


_NUMERICSETENTRY = _descriptor.Descriptor(
  name='NumericSetEntry',
  full_name='v1.model.NumericSetEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.NumericSetEntry.value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='label', full_name='v1.model.NumericSetEntry.label', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='label', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unit', full_name='v1.model.NumericSetEntry.unit', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='unit', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=154,
)


_NUMERICSET = _descriptor.Descriptor(
  name='NumericSet',
  full_name='v1.model.NumericSet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='numerics', full_name='v1.model.NumericSet.numerics', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='numerics', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=156,
  serialized_end=223,
)


_BIT = _descriptor.Descriptor(
  name='Bit',
  full_name='v1.model.Bit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='v1.model.Bit.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.Bit.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=225,
  serialized_end=270,
)


_BITSET = _descriptor.Descriptor(
  name='Bitset',
  full_name='v1.model.Bitset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bits', full_name='v1.model.Bitset.bits', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bits', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=272,
  serialized_end=315,
)


_TWIST = _descriptor.Descriptor(
  name='Twist',
  full_name='v1.model.Twist',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='linear', full_name='v1.model.Twist.linear', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='linear', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='angular', full_name='v1.model.Twist.angular', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='angular', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=317,
  serialized_end=412,
)


_TRANSFORM = _descriptor.Descriptor(
  name='Transform',
  full_name='v1.model.Transform',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='translation', full_name='v1.model.Transform.translation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='translation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rotation', full_name='v1.model.Transform.rotation', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rotation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=414,
  serialized_end=528,
)


_TRANSFORMFRAME = _descriptor.Descriptor(
  name='TransformFrame',
  full_name='v1.model.TransformFrame',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='parent_frame', full_name='v1.model.TransformFrame.parent_frame', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='parentFrame', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='child_frame', full_name='v1.model.TransformFrame.child_frame', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='childFrame', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transform', full_name='v1.model.TransformFrame.transform', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transform', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=531,
  serialized_end=666,
)


_VECTOR3 = _descriptor.Descriptor(
  name='Vector3',
  full_name='v1.model.Vector3',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='v1.model.Vector3.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='x', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='v1.model.Vector3.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='y', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='z', full_name='v1.model.Vector3.z', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='z', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=668,
  serialized_end=719,
)


_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='v1.model.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='v1.model.Point.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='x', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='v1.model.Point.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='y', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='z', full_name='v1.model.Point.z', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='z', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=721,
  serialized_end=770,
)


_QUATERNION = _descriptor.Descriptor(
  name='Quaternion',
  full_name='v1.model.Quaternion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='v1.model.Quaternion.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='x', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='v1.model.Quaternion.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='y', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='z', full_name='v1.model.Quaternion.z', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='z', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='w', full_name='v1.model.Quaternion.w', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='w', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=772,
  serialized_end=840,
)

_NUMERICSET.fields_by_name['numerics'].message_type = _NUMERICSETENTRY
_BITSET.fields_by_name['bits'].message_type = _BIT
_TWIST.fields_by_name['linear'].message_type = _VECTOR3
_TWIST.fields_by_name['angular'].message_type = _VECTOR3
_TRANSFORM.fields_by_name['translation'].message_type = _VECTOR3
_TRANSFORM.fields_by_name['rotation'].message_type = _QUATERNION
_TRANSFORMFRAME.fields_by_name['transform'].message_type = _TRANSFORM
DESCRIPTOR.message_types_by_name['Numeric'] = _NUMERIC
DESCRIPTOR.message_types_by_name['NumericSetEntry'] = _NUMERICSETENTRY
DESCRIPTOR.message_types_by_name['NumericSet'] = _NUMERICSET
DESCRIPTOR.message_types_by_name['Bit'] = _BIT
DESCRIPTOR.message_types_by_name['Bitset'] = _BITSET
DESCRIPTOR.message_types_by_name['Twist'] = _TWIST
DESCRIPTOR.message_types_by_name['Transform'] = _TRANSFORM
DESCRIPTOR.message_types_by_name['TransformFrame'] = _TRANSFORMFRAME
DESCRIPTOR.message_types_by_name['Vector3'] = _VECTOR3
DESCRIPTOR.message_types_by_name['Point'] = _POINT
DESCRIPTOR.message_types_by_name['Quaternion'] = _QUATERNION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Numeric = _reflection.GeneratedProtocolMessageType('Numeric', (_message.Message,), {
  'DESCRIPTOR' : _NUMERIC,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Numeric)
  })
_sym_db.RegisterMessage(Numeric)

NumericSetEntry = _reflection.GeneratedProtocolMessageType('NumericSetEntry', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICSETENTRY,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.NumericSetEntry)
  })
_sym_db.RegisterMessage(NumericSetEntry)

NumericSet = _reflection.GeneratedProtocolMessageType('NumericSet', (_message.Message,), {
  'DESCRIPTOR' : _NUMERICSET,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.NumericSet)
  })
_sym_db.RegisterMessage(NumericSet)

Bit = _reflection.GeneratedProtocolMessageType('Bit', (_message.Message,), {
  'DESCRIPTOR' : _BIT,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Bit)
  })
_sym_db.RegisterMessage(Bit)

Bitset = _reflection.GeneratedProtocolMessageType('Bitset', (_message.Message,), {
  'DESCRIPTOR' : _BITSET,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Bitset)
  })
_sym_db.RegisterMessage(Bitset)

Twist = _reflection.GeneratedProtocolMessageType('Twist', (_message.Message,), {
  'DESCRIPTOR' : _TWIST,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Twist)
  })
_sym_db.RegisterMessage(Twist)

Transform = _reflection.GeneratedProtocolMessageType('Transform', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORM,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Transform)
  })
_sym_db.RegisterMessage(Transform)

TransformFrame = _reflection.GeneratedProtocolMessageType('TransformFrame', (_message.Message,), {
  'DESCRIPTOR' : _TRANSFORMFRAME,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.TransformFrame)
  })
_sym_db.RegisterMessage(TransformFrame)

Vector3 = _reflection.GeneratedProtocolMessageType('Vector3', (_message.Message,), {
  'DESCRIPTOR' : _VECTOR3,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Vector3)
  })
_sym_db.RegisterMessage(Vector3)

Point = _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
  'DESCRIPTOR' : _POINT,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Point)
  })
_sym_db.RegisterMessage(Point)

Quaternion = _reflection.GeneratedProtocolMessageType('Quaternion', (_message.Message,), {
  'DESCRIPTOR' : _QUATERNION,
  '__module__' : 'protos.model.v1.math_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Quaternion)
  })
_sym_db.RegisterMessage(Quaternion)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import time
import sys
from threading import Lock
from typing import Dict, Optional
from formant.sdk.agent.v1.exceptions import handle_agent_exceptions
from formant.protos.model.v1 import navigation_pb2, datapoint_pb2
from formant.sdk.agent.v1.localization.types import (
    Odometry,
    Path,
    Goal,
    Map,
    PointCloud,
)

DEFAULT_POINTCLOUD_NAME = "defualt"


class LocalizationManager:
    def __init__(
        self,
        stream_name,  # type: str
        client,
        throttle_hz,  # type: float
    ):
        self._client = client
        self._stream_name = stream_name  # type: str
        self._last_sent_time = time.time()  # type: float
        self._lock = Lock()
        self._cached_odometry = None  # type: Optional[Odometry]
        self._cached_map = None  # type: Optional[Map]
        self._cached_point_clouds = {}  # type: Dict[str, PointCloud]
        self._cached_path = None  # type: Optional[Path]
        self._cached_goal = None  # type: Optional[Goal]

        self.ignore_throttled = client.ignore_throttled
        self.ignore_unavailable = client.ignore_unavailable

        if throttle_hz == 0:
            self._min_time_between_messages = float("inf")
        else:
            self._min_time_between_messages = 1.0 / throttle_hz
        self._tags = {}  # type: Dict

    def update_point_cloud(
        self,
        point_cloud,  # type: PointCloud
        cloud_name=DEFAULT_POINTCLOUD_NAME,  # type:str
    ):
        with self._lock:
            self._cached_point_clouds[cloud_name] = point_cloud
        self._try_send_localization()

    def invalidate_pointcloud(
        self, cloud_name=DEFAULT_POINTCLOUD_NAME  # type: str
    ):
        with self._lock:
            del self._cached_point_clouds[cloud_name]
        self._try_send_localization()

    def update_odometry(
        self, odometry  # type: Odometry
    ):
        with self._lock:
            self._cached_odometry = odometry
        self._try_send_localization()

    def update_map(
        self, map  # type: Map
    ):
        with self._lock:
            self._cached_map = map
        self._try_send_localization()

    def update_path(
        self, path  # type: Path
    ):
        with self._lock:
            self._cached_path = path
        self._try_send_localization()

    def update_goal(
        self, goal  # type: Goal
    ):
        with self._lock:
            self._cached_goal = goal
        self._try_send_localization()

    def set_tags(
        self, tags  # type: Dict
    ):
        with self._lock:
            self._tags = tags

    @handle_agent_exceptions
    def _try_send_localization(self):
        with self._lock:
            # Localization requires odometry
            if self._cached_odometry is None:
                return
            # Don't send messages too fast
            now = time.time()
            if now - self._last_sent_time < self._min_time_between_messages:
                return
            self._last_sent_time = now

            map = self._cached_map
            odometry = self._cached_odometry
            path = self._cached_path
            goal = self._cached_goal
            point_clouds = self._cached_point_clouds
            tags = self._tags

        localization = navigation_pb2.Localization(
            map=protected_to_proto(map),
            odometry=protected_to_proto(odometry),
            path=protected_to_proto(path),
            goal=protected_to_proto(goal),
        )
        if len(point_clouds.keys()) != 0:
            del localization.point_clouds[:]
            localization.point_clouds.extend(
                [point_cloud.to_proto() for point_cloud in point_clouds.values()]
            )

        self._client.agent_stub.PostData(
            datapoint_pb2.Datapoint(
                stream=self._stream_name,
                localization=localization,
                tags=tags,
                timestamp=int(time.time() * 1000),
            )
        )


def protected_to_proto(obj):
    if obj is None:
        return None
    return obj.to_proto()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/model/v1/event.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/model/v1/event.proto',
  package='v1.model',
  syntax='proto3',
  serialized_options=b'Z)github.com/FormantIO/genproto/go/v1/model',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1bprotos/model/v1/event.proto\x12\x08v1.model\"\xf7\x02\n\x05\x45vent\x12\x1c\n\ttimestamp\x18\x01 \x01(\x03R\ttimestamp\x12#\n\rend_timestamp\x18\x08 \x01(\x03R\x0c\x65ndTimestamp\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x1f\n\x0bstream_name\x18\x03 \x01(\tR\nstreamName\x12\x1f\n\x0bstream_type\x18\x04 \x01(\tR\nstreamType\x12\x31\n\x14notification_enabled\x18\x05 \x01(\x08R\x13notificationEnabled\x12.\n\x08severity\x18\t \x01(\x0e\x32\x12.v1.model.SeverityR\x08severity\x12-\n\x04tags\x18\x06 \x03(\x0b\x32\x19.v1.model.Event.TagsEntryR\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x07\x10\x08\"\x80\x01\n\x17\x41gentEventConfiguration\x12\x42\n\x0e\x65vent_triggers\x18\x01 \x03(\x0b\x32\x1b.v1.model.AgentEventTriggerR\reventTriggers\x12!\n\x0clast_updated\x18\x02 \x01(\x03R\x0blastUpdated\"\xf4\x02\n\x11\x41gentEventTrigger\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x16\n\x06stream\x18\x02 \x01(\tR\x06stream\x12\x1a\n\x08interval\x18\x03 \x01(\x03R\x08interval\x12\x45\n\x08presence\x18\x04 \x01(\x0b\x32\'.v1.model.PresenceEventTriggerConditionH\x00R\x08presence\x12H\n\tthreshold\x18\x05 \x01(\x0b\x32(.v1.model.ThresholdEventTriggerConditionH\x00R\tthreshold\x12<\n\x05regex\x18\x06 \x01(\x0b\x32$.v1.model.RegexEventTriggerConditionH\x00R\x05regex\x12?\n\x06\x62itset\x18\x07 \x01(\x0b\x32%.v1.model.BitsetEventTriggerConditionH\x00R\x06\x62itsetB\x0b\n\tcondition\"\x1f\n\x1dPresenceEventTriggerCondition\"o\n\x1eThresholdEventTriggerCondition\x12\x14\n\x05value\x18\x01 \x01(\x01R\x05value\x12\x37\n\x08operator\x18\x02 \x01(\x0e\x32\x1b.v1.model.ThresholdOperatorR\x08operator\"2\n\x1aRegexEventTriggerCondition\x12\x14\n\x05value\x18\x01 \x01(\tR\x05value\"\x95\x01\n\x1b\x42itsetEventTriggerCondition\x12@\n\x0e\x62it_conditions\x18\x01 \x03(\x0b\x32\x19.v1.model.BitsetConditionR\rbitConditions\x12\x34\n\x08operator\x18\x02 \x01(\x0e\x32\x18.v1.model.BitsetOperatorR\x08operator\"M\n\x0f\x42itsetCondition\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x12\n\x04true\x18\x02 \x01(\x08R\x04true\x12\x14\n\x05\x66\x61lse\x18\x03 \x01(\x08R\x05\x66\x61lse*:\n\x08Severity\x12\x08\n\x04INFO\x10\x00\x12\x0b\n\x07WARNING\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0c\n\x08\x43RITICAL\x10\x03*F\n\x11ThresholdOperator\x12\x06\n\x02LT\x10\x00\x12\x07\n\x03LTE\x10\x01\x12\x06\n\x02GT\x10\x02\x12\x07\n\x03GTE\x10\x03\x12\x06\n\x02\x45Q\x10\x04\x12\x07\n\x03NEQ\x10\x05*\"\n\x0e\x42itsetOperator\x12\x07\n\x03\x41NY\x10\x00\x12\x07\n\x03\x41LL\x10\x01\x42+Z)github.com/FormantIO/genproto/go/v1/modelb\x06proto3'
)

_SEVERITY = _descriptor.EnumDescriptor(
  name='Severity',
  full_name='v1.model.Severity',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INFO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1354,
  serialized_end=1412,
)
_sym_db.RegisterEnumDescriptor(_SEVERITY)

Severity = enum_type_wrapper.EnumTypeWrapper(_SEVERITY)
_THRESHOLDOPERATOR = _descriptor.EnumDescriptor(
  name='ThresholdOperator',
  full_name='v1.model.ThresholdOperator',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LTE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GTE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EQ', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NEQ', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1414,
  serialized_end=1484,
)
_sym_db.RegisterEnumDescriptor(_THRESHOLDOPERATOR)

ThresholdOperator = enum_type_wrapper.EnumTypeWrapper(_THRESHOLDOPERATOR)
_BITSETOPERATOR = _descriptor.EnumDescriptor(
  name='BitsetOperator',
  full_name='v1.model.BitsetOperator',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ANY', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ALL', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1486,
  serialized_end=1520,
)
_sym_db.RegisterEnumDescriptor(_BITSETOPERATOR)

BitsetOperator = enum_type_wrapper.EnumTypeWrapper(_BITSETOPERATOR)
INFO = 0
WARNING = 1
ERROR = 2
CRITICAL = 3
LT = 0
LTE = 1
GT = 2
GTE = 3
EQ = 4
NEQ = 5
ANY = 0
ALL = 1



_EVENT_TAGSENTRY = _descriptor.Descriptor(
  name='TagsEntry',
  full_name='v1.model.Event.TagsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='v1.model.Event.TagsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.Event.TagsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=356,
  serialized_end=411,
)

_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='v1.model.Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='v1.model.Event.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_timestamp', full_name='v1.model.Event.end_timestamp', index=1,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='endTimestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='v1.model.Event.message', index=2,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='message', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stream_name', full_name='v1.model.Event.stream_name', index=3,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stream_type', full_name='v1.model.Event.stream_type', index=4,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='notification_enabled', full_name='v1.model.Event.notification_enabled', index=5,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='notificationEnabled', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='severity', full_name='v1.model.Event.severity', index=6,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='severity', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags', full_name='v1.model.Event.tags', index=7,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='tags', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EVENT_TAGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=42,
  serialized_end=417,
)


_AGENTEVENTCONFIGURATION = _descriptor.Descriptor(
  name='AgentEventConfiguration',
  full_name='v1.model.AgentEventConfiguration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_triggers', full_name='v1.model.AgentEventConfiguration.event_triggers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='eventTriggers', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_updated', full_name='v1.model.AgentEventConfiguration.last_updated', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='lastUpdated', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=420,
  serialized_end=548,
)


_AGENTEVENTTRIGGER = _descriptor.Descriptor(
  name='AgentEventTrigger',
  full_name='v1.model.AgentEventTrigger',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='v1.model.AgentEventTrigger.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stream', full_name='v1.model.AgentEventTrigger.stream', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='stream', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='interval', full_name='v1.model.AgentEventTrigger.interval', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='interval', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='presence', full_name='v1.model.AgentEventTrigger.presence', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='presence', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='threshold', full_name='v1.model.AgentEventTrigger.threshold', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='threshold', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='regex', full_name='v1.model.AgentEventTrigger.regex', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='regex', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bitset', full_name='v1.model.AgentEventTrigger.bitset', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bitset', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='condition', full_name='v1.model.AgentEventTrigger.condition',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=551,
  serialized_end=923,
)


_PRESENCEEVENTTRIGGERCONDITION = _descriptor.Descriptor(
  name='PresenceEventTriggerCondition',
  full_name='v1.model.PresenceEventTriggerCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=925,
  serialized_end=956,
)


_THRESHOLDEVENTTRIGGERCONDITION = _descriptor.Descriptor(
  name='ThresholdEventTriggerCondition',
  full_name='v1.model.ThresholdEventTriggerCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.ThresholdEventTriggerCondition.value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operator', full_name='v1.model.ThresholdEventTriggerCondition.operator', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='operator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=958,
  serialized_end=1069,
)


_REGEXEVENTTRIGGERCONDITION = _descriptor.Descriptor(
  name='RegexEventTriggerCondition',
  full_name='v1.model.RegexEventTriggerCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.RegexEventTriggerCondition.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1071,
  serialized_end=1121,
)


_BITSETEVENTTRIGGERCONDITION = _descriptor.Descriptor(
  name='BitsetEventTriggerCondition',
  full_name='v1.model.BitsetEventTriggerCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bit_conditions', full_name='v1.model.BitsetEventTriggerCondition.bit_conditions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bitConditions', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operator', full_name='v1.model.BitsetEventTriggerCondition.operator', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='operator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1124,
  serialized_end=1273,
)


_BITSETCONDITION = _descriptor.Descriptor(
  name='BitsetCondition',
  full_name='v1.model.BitsetCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='v1.model.BitsetCondition.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='true', full_name='v1.model.BitsetCondition.true', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='true', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='false', full_name='v1.model.BitsetCondition.false', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='false', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1275,
  serialized_end=1352,
)

_EVENT_TAGSENTRY.containing_type = _EVENT
_EVENT.fields_by_name['severity'].enum_type = _SEVERITY
_EVENT.fields_by_name['tags'].message_type = _EVENT_TAGSENTRY
_AGENTEVENTCONFIGURATION.fields_by_name['event_triggers'].message_type = _AGENTEVENTTRIGGER
_AGENTEVENTTRIGGER.fields_by_name['presence'].message_type = _PRESENCEEVENTTRIGGERCONDITION
_AGENTEVENTTRIGGER.fields_by_name['threshold'].message_type = _THRESHOLDEVENTTRIGGERCONDITION
_AGENTEVENTTRIGGER.fields_by_name['regex'].message_type = _REGEXEVENTTRIGGERCONDITION
_AGENTEVENTTRIGGER.fields_by_name['bitset'].message_type = _BITSETEVENTTRIGGERCONDITION
_AGENTEVENTTRIGGER.oneofs_by_name['condition'].fields.append(
  _AGENTEVENTTRIGGER.fields_by_name['presence'])
_AGENTEVENTTRIGGER.fields_by_name['presence'].containing_oneof = _AGENTEVENTTRIGGER.oneofs_by_name['condition']
_AGENTEVENTTRIGGER.oneofs_by_name['condition'].fields.append(
  _AGENTEVENTTRIGGER.fields_by_name['threshold'])
_AGENTEVENTTRIGGER.fields_by_name['threshold'].containing_oneof = _AGENTEVENTTRIGGER.oneofs_by_name['condition']
_AGENTEVENTTRIGGER.oneofs_by_name['condition'].fields.append(
  _AGENTEVENTTRIGGER.fields_by_name['regex'])
_AGENTEVENTTRIGGER.fields_by_name['regex'].containing_oneof = _AGENTEVENTTRIGGER.oneofs_by_name['condition']
_AGENTEVENTTRIGGER.oneofs_by_name['condition'].fields.append(
  _AGENTEVENTTRIGGER.fields_by_name['bitset'])
_AGENTEVENTTRIGGER.fields_by_name['bitset'].containing_oneof = _AGENTEVENTTRIGGER.oneofs_by_name['condition']
_THRESHOLDEVENTTRIGGERCONDITION.fields_by_name['operator'].enum_type = _THRESHOLDOPERATOR
_BITSETEVENTTRIGGERCONDITION.fields_by_name['bit_conditions'].message_type = _BITSETCONDITION
_BITSETEVENTTRIGGERCONDITION.fields_by_name['operator'].enum_type = _BITSETOPERATOR
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.message_types_by_name['AgentEventConfiguration'] = _AGENTEVENTCONFIGURATION
DESCRIPTOR.message_types_by_name['AgentEventTrigger'] = _AGENTEVENTTRIGGER
DESCRIPTOR.message_types_by_name['PresenceEventTriggerCondition'] = _PRESENCEEVENTTRIGGERCONDITION
DESCRIPTOR.message_types_by_name['ThresholdEventTriggerCondition'] = _THRESHOLDEVENTTRIGGERCONDITION
DESCRIPTOR.message_types_by_name['RegexEventTriggerCondition'] = _REGEXEVENTTRIGGERCONDITION
DESCRIPTOR.message_types_by_name['BitsetEventTriggerCondition'] = _BITSETEVENTTRIGGERCONDITION
DESCRIPTOR.message_types_by_name['BitsetCondition'] = _BITSETCONDITION
DESCRIPTOR.enum_types_by_name['Severity'] = _SEVERITY
DESCRIPTOR.enum_types_by_name['ThresholdOperator'] = _THRESHOLDOPERATOR
DESCRIPTOR.enum_types_by_name['BitsetOperator'] = _BITSETOPERATOR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_TAGSENTRY,
    '__module__' : 'protos.model.v1.event_pb2'
    # @@protoc_insertion_point(class_scope:v1.model.Event.TagsEntry)
    })
  ,
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Event)
  })
_sym_db.RegisterMessage(Event)
_sym_db.RegisterMessage(Event.TagsEntry)

AgentEventConfiguration = _reflection.GeneratedProtocolMessageType('AgentEventConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _AGENTEVENTCONFIGURATION,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.AgentEventConfiguration)
  })
_sym_db.RegisterMessage(AgentEventConfiguration)

AgentEventTrigger = _reflection.GeneratedProtocolMessageType('AgentEventTrigger', (_message.Message,), {
  'DESCRIPTOR' : _AGENTEVENTTRIGGER,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.AgentEventTrigger)
  })
_sym_db.RegisterMessage(AgentEventTrigger)

PresenceEventTriggerCondition = _reflection.GeneratedProtocolMessageType('PresenceEventTriggerCondition', (_message.Message,), {
  'DESCRIPTOR' : _PRESENCEEVENTTRIGGERCONDITION,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.PresenceEventTriggerCondition)
  })
_sym_db.RegisterMessage(PresenceEventTriggerCondition)

ThresholdEventTriggerCondition = _reflection.GeneratedProtocolMessageType('ThresholdEventTriggerCondition', (_message.Message,), {
  'DESCRIPTOR' : _THRESHOLDEVENTTRIGGERCONDITION,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.ThresholdEventTriggerCondition)
  })
_sym_db.RegisterMessage(ThresholdEventTriggerCondition)

RegexEventTriggerCondition = _reflection.GeneratedProtocolMessageType('RegexEventTriggerCondition', (_message.Message,), {
  'DESCRIPTOR' : _REGEXEVENTTRIGGERCONDITION,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.RegexEventTriggerCondition)
  })
_sym_db.RegisterMessage(RegexEventTriggerCondition)

BitsetEventTriggerCondition = _reflection.GeneratedProtocolMessageType('BitsetEventTriggerCondition', (_message.Message,), {
  'DESCRIPTOR' : _BITSETEVENTTRIGGERCONDITION,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.BitsetEventTriggerCondition)
  })
_sym_db.RegisterMessage(BitsetEventTriggerCondition)

BitsetCondition = _reflection.GeneratedProtocolMessageType('BitsetCondition', (_message.Message,), {
  'DESCRIPTOR' : _BITSETCONDITION,
  '__module__' : 'protos.model.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.BitsetCondition)
  })
_sym_db.RegisterMessage(BitsetCondition)


DESCRIPTOR._options = None
_EVENT_TAGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/model/v1/intervention.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from formant.protos.model.v1 import media_pb2 as protos_dot_model_dot_v1_dot_media__pb2
from formant.protos.model.v1 import event_pb2 as protos_dot_model_dot_v1_dot_event__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/model/v1/intervention.proto',
  package='v1.model',
  syntax='proto3',
  serialized_options=b'Z)github.com/FormantIO/genproto/go/v1/model',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"protos/model/v1/intervention.proto\x12\x08v1.model\x1a\x1bprotos/model/v1/media.proto\x1a\x1bprotos/model/v1/event.proto\"\x84\x04\n\x13InterventionRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n\ttimestamp\x18\x02 \x01(\x03R\ttimestamp\x12.\n\x08severity\x18\x03 \x01(\x0e\x32\x12.v1.model.SeverityR\x08severity\x12I\n\x11selection_request\x18\x04 \x01(\x0b\x32\x1a.v1.model.SelectionRequestH\x00R\x10selectionRequest\x12\x46\n\x10labeling_request\x18\x05 \x01(\x0b\x32\x19.v1.model.LabelingRequestH\x00R\x0flabelingRequest\x12@\n\x0eteleop_request\x18\x08 \x01(\x0b\x32\x17.v1.model.TeleopRequestH\x00R\rteleopRequest\x12;\n\x04tags\x18\x06 \x03(\x0b\x32\'.v1.model.InterventionRequest.TagsEntryR\x04tags\x12<\n\tresponses\x18\x07 \x03(\x0b\x32\x1e.v1.model.InterventionResponseR\tresponses\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x06\n\x04\x64\x61ta\"\xc9\x02\n\x14InterventionResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1d\n\nrequest_id\x18\x02 \x01(\tR\trequestId\x12\x1c\n\ttimestamp\x18\x03 \x01(\x03R\ttimestamp\x12L\n\x12selection_response\x18\x04 \x01(\x0b\x32\x1b.v1.model.SelectionResponseH\x00R\x11selectionResponse\x12I\n\x11labeling_response\x18\x05 \x01(\x0b\x32\x1a.v1.model.LabelingResponseH\x00R\x10labelingResponse\x12\x43\n\x0fteleop_response\x18\x06 \x01(\x0b\x32\x18.v1.model.TeleopResponseH\x00R\x0eteleopResponseB\x06\n\x04\x64\x61ta\"@\n\x05Label\x12\x14\n\x05value\x18\x01 \x01(\tR\x05value\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tR\x0b\x64isplayName\"V\n\x0eLabeledPolygon\x12,\n\x08vertices\x18\x01 \x03(\x0b\x32\x10.v1.model.VertexR\x08vertices\x12\x16\n\x06labels\x18\x02 \x03(\tR\x06labels\"$\n\x06Vertex\x12\x0c\n\x01x\x18\x01 \x01(\x01R\x01x\x12\x0c\n\x01y\x18\x02 \x01(\x01R\x01y\"\xc7\x01\n\x0fLabelingRequest\x12\x14\n\x05title\x18\x01 \x01(\tR\x05title\x12 \n\x0binstruction\x18\x02 \x01(\tR\x0binstruction\x12%\n\x05image\x18\x03 \x01(\x0b\x32\x0f.v1.model.ImageR\x05image\x12\'\n\x06labels\x18\x04 \x03(\x0b\x32\x0f.v1.model.LabelR\x06labels\x12,\n\x04hint\x18\x05 \x03(\x0b\x32\x18.v1.model.LabeledPolygonR\x04hint\"B\n\x10LabelingResponse\x12.\n\x05value\x18\x01 \x03(\x0b\x32\x18.v1.model.LabeledPolygonR\x05value\"\xa9\x01\n\x10SelectionRequest\x12\x14\n\x05title\x18\x01 \x01(\tR\x05title\x12\'\n\x05image\x18\x02 \x01(\x0b\x32\x0f.v1.model.ImageH\x00R\x05image\x12 \n\x0binstruction\x18\x03 \x01(\tR\x0binstruction\x12\x18\n\x07options\x18\x04 \x03(\tR\x07options\x12\x12\n\x04hint\x18\x05 \x01(\rR\x04hintB\x06\n\x04\x64\x61ta\")\n\x11SelectionResponse\x12\x14\n\x05value\x18\x01 \x01(\rR\x05value\"1\n\rTeleopRequest\x12 \n\x0binstruction\x18\x01 \x01(\tR\x0binstruction\"<\n\x0eTeleopResponse\x12\x14\n\x05state\x18\x01 \x01(\tR\x05state\x12\x14\n\x05notes\x18\x02 \x01(\tR\x05notesB+Z)github.com/FormantIO/genproto/go/v1/modelb\x06proto3'
  ,
  dependencies=[protos_dot_model_dot_v1_dot_media__pb2.DESCRIPTOR,protos_dot_model_dot_v1_dot_event__pb2.DESCRIPTOR,])




_INTERVENTIONREQUEST_TAGSENTRY = _descriptor.Descriptor(
  name='TagsEntry',
  full_name='v1.model.InterventionRequest.TagsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='v1.model.InterventionRequest.TagsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.InterventionRequest.TagsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=560,
  serialized_end=615,
)

_INTERVENTIONREQUEST = _descriptor.Descriptor(
  name='InterventionRequest',
  full_name='v1.model.InterventionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='v1.model.InterventionRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='v1.model.InterventionRequest.timestamp', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='severity', full_name='v1.model.InterventionRequest.severity', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='severity', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selection_request', full_name='v1.model.InterventionRequest.selection_request', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='selectionRequest', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labeling_request', full_name='v1.model.InterventionRequest.labeling_request', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='labelingRequest', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='teleop_request', full_name='v1.model.InterventionRequest.teleop_request', index=5,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='teleopRequest', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags', full_name='v1.model.InterventionRequest.tags', index=6,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='tags', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='responses', full_name='v1.model.InterventionRequest.responses', index=7,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='responses', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_INTERVENTIONREQUEST_TAGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='v1.model.InterventionRequest.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=107,
  serialized_end=623,
)


_INTERVENTIONRESPONSE = _descriptor.Descriptor(
  name='InterventionResponse',
  full_name='v1.model.InterventionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='v1.model.InterventionResponse.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='request_id', full_name='v1.model.InterventionResponse.request_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='requestId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='v1.model.InterventionResponse.timestamp', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selection_response', full_name='v1.model.InterventionResponse.selection_response', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='selectionResponse', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labeling_response', full_name='v1.model.InterventionResponse.labeling_response', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='labelingResponse', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='teleop_response', full_name='v1.model.InterventionResponse.teleop_response', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='teleopResponse', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='v1.model.InterventionResponse.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=626,
  serialized_end=955,
)


_LABEL = _descriptor.Descriptor(
  name='Label',
  full_name='v1.model.Label',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.Label.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='display_name', full_name='v1.model.Label.display_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='displayName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=957,
  serialized_end=1021,
)


_LABELEDPOLYGON = _descriptor.Descriptor(
  name='LabeledPolygon',
  full_name='v1.model.LabeledPolygon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='vertices', full_name='v1.model.LabeledPolygon.vertices', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='vertices', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labels', full_name='v1.model.LabeledPolygon.labels', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='labels', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1023,
  serialized_end=1109,
)


_VERTEX = _descriptor.Descriptor(
  name='Vertex',
  full_name='v1.model.Vertex',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='v1.model.Vertex.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='x', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='v1.model.Vertex.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='y', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1111,
  serialized_end=1147,
)


_LABELINGREQUEST = _descriptor.Descriptor(
  name='LabelingRequest',
  full_name='v1.model.LabelingRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='v1.model.LabelingRequest.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='title', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instruction', full_name='v1.model.LabelingRequest.instruction', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='instruction', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='image', full_name='v1.model.LabelingRequest.image', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='image', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='labels', full_name='v1.model.LabelingRequest.labels', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='labels', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hint', full_name='v1.model.LabelingRequest.hint', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='hint', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1150,
  serialized_end=1349,
)


_LABELINGRESPONSE = _descriptor.Descriptor(
  name='LabelingResponse',
  full_name='v1.model.LabelingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.LabelingResponse.value', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1351,
  serialized_end=1417,
)


_SELECTIONREQUEST = _descriptor.Descriptor(
  name='SelectionRequest',
  full_name='v1.model.SelectionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='v1.model.SelectionRequest.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='title', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='image', full_name='v1.model.SelectionRequest.image', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='image', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instruction', full_name='v1.model.SelectionRequest.instruction', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='instruction', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='options', full_name='v1.model.SelectionRequest.options', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='options', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hint', full_name='v1.model.SelectionRequest.hint', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='hint', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='v1.model.SelectionRequest.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1420,
  serialized_end=1589,
)


_SELECTIONRESPONSE = _descriptor.Descriptor(
  name='SelectionResponse',
  full_name='v1.model.SelectionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='v1.model.SelectionResponse.value', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1591,
  serialized_end=1632,
)


_TELEOPREQUEST = _descriptor.Descriptor(
  name='TeleopRequest',
  full_name='v1.model.TeleopRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='instruction', full_name='v1.model.TeleopRequest.instruction', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='instruction', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1634,
  serialized_end=1683,
)


_TELEOPRESPONSE = _descriptor.Descriptor(
  name='TeleopResponse',
  full_name='v1.model.TeleopResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='v1.model.TeleopResponse.state', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='state', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='notes', full_name='v1.model.TeleopResponse.notes', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='notes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1685,
  serialized_end=1745,
)

_INTERVENTIONREQUEST_TAGSENTRY.containing_type = _INTERVENTIONREQUEST
_INTERVENTIONREQUEST.fields_by_name['severity'].enum_type = protos_dot_model_dot_v1_dot_event__pb2._SEVERITY
_INTERVENTIONREQUEST.fields_by_name['selection_request'].message_type = _SELECTIONREQUEST
_INTERVENTIONREQUEST.fields_by_name['labeling_request'].message_type = _LABELINGREQUEST
_INTERVENTIONREQUEST.fields_by_name['teleop_request'].message_type = _TELEOPREQUEST
_INTERVENTIONREQUEST.fields_by_name['tags'].message_type = _INTERVENTIONREQUEST_TAGSENTRY
_INTERVENTIONREQUEST.fields_by_name['responses'].message_type = _INTERVENTIONRESPONSE
_INTERVENTIONREQUEST.oneofs_by_name['data'].fields.append(
  _INTERVENTIONREQUEST.fields_by_name['selection_request'])
_INTERVENTIONREQUEST.fields_by_name['selection_request'].containing_oneof = _INTERVENTIONREQUEST.oneofs_by_name['data']
_INTERVENTIONREQUEST.oneofs_by_name['data'].fields.append(
  _INTERVENTIONREQUEST.fields_by_name['labeling_request'])
_INTERVENTIONREQUEST.fields_by_name['labeling_request'].containing_oneof = _INTERVENTIONREQUEST.oneofs_by_name['data']
_INTERVENTIONREQUEST.oneofs_by_name['data'].fields.append(
  _INTERVENTIONREQUEST.fields_by_name['teleop_request'])
_INTERVENTIONREQUEST.fields_by_name['teleop_request'].containing_oneof = _INTERVENTIONREQUEST.oneofs_by_name['data']
_INTERVENTIONRESPONSE.fields_by_name['selection_response'].message_type = _SELECTIONRESPONSE
_INTERVENTIONRESPONSE.fields_by_name['labeling_response'].message_type = _LABELINGRESPONSE
_INTERVENTIONRESPONSE.fields_by_name['teleop_response'].message_type = _TELEOPRESPONSE
_INTERVENTIONRESPONSE.oneofs_by_name['data'].fields.append(
  _INTERVENTIONRESPONSE.fields_by_name['selection_response'])
_INTERVENTIONRESPONSE.fields_by_name['selection_response'].containing_oneof = _INTERVENTIONRESPONSE.oneofs_by_name['data']
_INTERVENTIONRESPONSE.oneofs_by_name['data'].fields.append(
  _INTERVENTIONRESPONSE.fields_by_name['labeling_response'])
_INTERVENTIONRESPONSE.fields_by_name['labeling_response'].containing_oneof = _INTERVENTIONRESPONSE.oneofs_by_name['data']
_INTERVENTIONRESPONSE.oneofs_by_name['data'].fields.append(
  _INTERVENTIONRESPONSE.fields_by_name['teleop_response'])
_INTERVENTIONRESPONSE.fields_by_name['teleop_response'].containing_oneof = _INTERVENTIONRESPONSE.oneofs_by_name['data']
_LABELEDPOLYGON.fields_by_name['vertices'].message_type = _VERTEX
_LABELINGREQUEST.fields_by_name['image'].message_type = protos_dot_model_dot_v1_dot_media__pb2._IMAGE
_LABELINGREQUEST.fields_by_name['labels'].message_type = _LABEL
_LABELINGREQUEST.fields_by_name['hint'].message_type = _LABELEDPOLYGON
_LABELINGRESPONSE.fields_by_name['value'].message_type = _LABELEDPOLYGON
_SELECTIONREQUEST.fields_by_name['image'].message_type = protos_dot_model_dot_v1_dot_media__pb2._IMAGE
_SELECTIONREQUEST.oneofs_by_name['data'].fields.append(
  _SELECTIONREQUEST.fields_by_name['image'])
_SELECTIONREQUEST.fields_by_name['image'].containing_oneof = _SELECTIONREQUEST.oneofs_by_name['data']
DESCRIPTOR.message_types_by_name['InterventionRequest'] = _INTERVENTIONREQUEST
DESCRIPTOR.message_types_by_name['InterventionResponse'] = _INTERVENTIONRESPONSE
DESCRIPTOR.message_types_by_name['Label'] = _LABEL
DESCRIPTOR.message_types_by_name['LabeledPolygon'] = _LABELEDPOLYGON
DESCRIPTOR.message_types_by_name['Vertex'] = _VERTEX
DESCRIPTOR.message_types_by_name['LabelingRequest'] = _LABELINGREQUEST
DESCRIPTOR.message_types_by_name['LabelingResponse'] = _LABELINGRESPONSE
DESCRIPTOR.message_types_by_name['SelectionRequest'] = _SELECTIONREQUEST
DESCRIPTOR.message_types_by_name['SelectionResponse'] = _SELECTIONRESPONSE
DESCRIPTOR.message_types_by_name['TeleopRequest'] = _TELEOPREQUEST
DESCRIPTOR.message_types_by_name['TeleopResponse'] = _TELEOPRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

InterventionRequest = _reflection.GeneratedProtocolMessageType('InterventionRequest', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _INTERVENTIONREQUEST_TAGSENTRY,
    '__module__' : 'protos.model.v1.intervention_pb2'
    # @@protoc_insertion_point(class_scope:v1.model.InterventionRequest.TagsEntry)
    })
  ,
  'DESCRIPTOR' : _INTERVENTIONREQUEST,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.InterventionRequest)
  })
_sym_db.RegisterMessage(InterventionRequest)
_sym_db.RegisterMessage(InterventionRequest.TagsEntry)

InterventionResponse = _reflection.GeneratedProtocolMessageType('InterventionResponse', (_message.Message,), {
  'DESCRIPTOR' : _INTERVENTIONRESPONSE,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.InterventionResponse)
  })
_sym_db.RegisterMessage(InterventionResponse)

Label = _reflection.GeneratedProtocolMessageType('Label', (_message.Message,), {
  'DESCRIPTOR' : _LABEL,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Label)
  })
_sym_db.RegisterMessage(Label)

LabeledPolygon = _reflection.GeneratedProtocolMessageType('LabeledPolygon', (_message.Message,), {
  'DESCRIPTOR' : _LABELEDPOLYGON,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.LabeledPolygon)
  })
_sym_db.RegisterMessage(LabeledPolygon)

Vertex = _reflection.GeneratedProtocolMessageType('Vertex', (_message.Message,), {
  'DESCRIPTOR' : _VERTEX,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.Vertex)
  })
_sym_db.RegisterMessage(Vertex)

LabelingRequest = _reflection.GeneratedProtocolMessageType('LabelingRequest', (_message.Message,), {
  'DESCRIPTOR' : _LABELINGREQUEST,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.LabelingRequest)
  })
_sym_db.RegisterMessage(LabelingRequest)

LabelingResponse = _reflection.GeneratedProtocolMessageType('LabelingResponse', (_message.Message,), {
  'DESCRIPTOR' : _LABELINGRESPONSE,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.LabelingResponse)
  })
_sym_db.RegisterMessage(LabelingResponse)

SelectionRequest = _reflection.GeneratedProtocolMessageType('SelectionRequest', (_message.Message,), {
  'DESCRIPTOR' : _SELECTIONREQUEST,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.SelectionRequest)
  })
_sym_db.RegisterMessage(SelectionRequest)

SelectionResponse = _reflection.GeneratedProtocolMessageType('SelectionResponse', (_message.Message,), {
  'DESCRIPTOR' : _SELECTIONRESPONSE,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.SelectionResponse)
  })
_sym_db.RegisterMessage(SelectionResponse)

TeleopRequest = _reflection.GeneratedProtocolMessageType('TeleopRequest', (_message.Message,), {
  'DESCRIPTOR' : _TELEOPREQUEST,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.TeleopRequest)
  })
_sym_db.RegisterMessage(TeleopRequest)

TeleopResponse = _reflection.GeneratedProtocolMessageType('TeleopResponse', (_message.Message,), {
  'DESCRIPTOR' : _TELEOPRESPONSE,
  '__module__' : 'protos.model.v1.intervention_pb2'
  # @@protoc_insertion_point(class_scope:v1.model.TeleopResponse)
  })
_sym_db.RegisterMessage(TeleopResponse)


DESCRIPTOR._options = None
_INTERVENTIONREQUEST_TAGSENTRY._options = None
# @@protoc_insertion_point(module_scope)



from typing import Dict
from .annotation_template import AnnotationTemplate


class AnnotationTemplateListResult:
    def __init__(
        self,
        annotation_template_list_result,  # type: Dict
        enabled_only=True,
    ):
        self._template_map = {
            template["name"]: AnnotationTemplate(template)
            for template in annotation_template_list_result["items"]
        }
        self._enabled_only = enabled_only

    def get_by_name(
        self, name  # type:str
    ):
        template = self._template_map.get(name, None)
        if template is None or (not template.enabled and self._enabled_only):
            return None
        return template

    def get_templates(self):
        return list(
            filter(
                lambda template: template.enabled if self._enabled_only else True,
                self._template_map.values(),
            )
        )



import numpy as np


def _get_fields_and_offsets(dt, offset=0):
    # counts up elements in subarrays, including nested subarrays, and returns
    # base dtype and count
    def count_elem(dt):
        count = 1
        while dt.shape != ():
            for size in dt.shape:
                count *= size
            dt = dt.base
        return dt, count

    fields = []
    for name in dt.names:
        field = dt.fields[name]
        f_dt, f_offset = field[0], field[1]
        f_dt, n = count_elem(f_dt)

        if f_dt.names is None:
            fields.append((np.dtype((f_dt, (n,))), n, f_offset + offset))
        else:
            subfields = _get_fields_and_offsets(f_dt, f_offset + offset)
            size = f_dt.itemsize

            for i in range(n):
                if i == 0:
                    # optimization: avoid list comprehension if no subarray
                    fields.extend(subfields)
                else:
                    fields.extend([(d, c, o + i * size) for d, c, o in subfields])
    return fields


def structured_to_unstructured(arr, dtype=None, copy=False, casting="unsafe"):
    if arr.dtype.names is None:
        raise ValueError("arr must be a structured array")

    fields = _get_fields_and_offsets(arr.dtype)
    n_fields = len(fields)
    if n_fields == 0 and dtype is None:
        raise ValueError("arr has no fields. Unable to guess dtype")
    elif n_fields == 0:
        # too many bugs elsewhere for this to work now
        raise NotImplementedError("arr with no fields is not supported")

    dts, counts, offsets = zip(*fields)
    names = ["f{}".format(n) for n in range(n_fields)]

    if dtype is None:
        out_dtype = np.result_type(*[dt.base for dt in dts])
    else:
        out_dtype = dtype

    # Use a series of views and casts to convert to an unstructured array:

    # first view using flattened fields (doesn't work for object arrays)
    # Note: dts may include a shape for subarrays
    flattened_fields = np.dtype(
        {
            "names": names,
            "formats": dts,
            "offsets": offsets,
            "itemsize": arr.dtype.itemsize,
        }
    )
    arr = arr.view(flattened_fields)

    # next cast to a packed format with all fields converted to new dtype
    packed_fields = np.dtype(
        {"names": names, "formats": [(out_dtype, dt.shape) for dt in dts]}
    )
    arr = arr.astype(packed_fields, copy=copy, casting=casting)

    # finally is it safe to view the packed fields as the unstructured type
    return arr.view((out_dtype, (sum(counts),)))


def unstructured_to_structured(
    arr, dtype=None, names=None, align=False, copy=False, casting="unsafe"
):
    if arr.shape == ():
        raise ValueError("arr must have at least one dimension")
    n_elem = arr.shape[-1]
    if n_elem == 0:
        # too many bugs elsewhere for this to work now
        raise NotImplementedError("last axis with size 0 is not supported")

    if dtype is None:
        if names is None:
            names = ["f{}".format(n) for n in range(n_elem)]
        out_dtype = np.dtype([(n, arr.dtype) for n in names], align=align)
        fields = _get_fields_and_offsets(out_dtype)
        dts, counts, offsets = zip(*fields)
    else:
        if names is not None:
            raise ValueError("don't supply both dtype and names")
        # sanity check of the input dtype
        fields = _get_fields_and_offsets(dtype)
        if len(fields) == 0:
            dts, counts, offsets = [], [], []
        else:
            dts, counts, offsets = zip(*fields)

        if n_elem != sum(counts):
            raise ValueError(
                "The length of the last dimension of arr must "
                "be equal to the number of fields in dtype"
            )
        out_dtype = dtype
        if align and not out_dtype.isalignedstruct:
            raise ValueError("align was True but dtype is not aligned")

    names = ["f{}".format(n) for n in range(len(fields))]

    # Use a series of views and casts to convert to a structured array:

    # first view as a packed structured array of one dtype
    packed_fields = np.dtype(
        {"names": names, "formats": [(arr.dtype, dt.shape) for dt in dts]}
    )
    arr = np.ascontiguousarray(arr).view(packed_fields)

    # next cast to an unpacked but flattened format with varied dtypes
    flattened_fields = np.dtype(
        {
            "names": names,
            "formats": dts,
            "offsets": offsets,
            "itemsize": out_dtype.itemsize,
        }
    )
    arr = arr.astype(flattened_fields, copy=copy, casting=casting)

    # finally view as the final nested dtype and remove the last axis
    return arr.view(out_dtype)[..., 0]



import re
from typing import Union, Dict, List

messagePathRegex = re.compile(
    r"^([a-zA-Z])([0-9a-zA-Z_])*(\[[0-9]+\])?(\.([a-zA-Z])([0-9a-zA-Z_])*(\[[0-9]+\])?)*"
)


def get_message_path_value(
    message,  # type: Union[Dict, object, List]
    messagePath,  # type: str
    raise_error=False,  # type: bool
):
    """
    Returns the message value at the end of the provided message path
    """
    if not messagePathRegex.fullmatch(messagePath):
        raise ValueError("Invalid message path in configuration: " + messagePath)

    # parse the message path
    steps = []
    for substring in messagePath.split("."):
        indexStart = None
        try:
            indexStart = substring.index("[")
        except ValueError:
            pass

        if indexStart:
            if indexStart == 0:
                steps.append(("indexing", substring))
            else:
                steps.append(("attribute", substring[:indexStart]))
                steps.append(("indexing", substring[indexStart:]))
        else:
            steps.append(("attribute", substring))

    # index based on the message path
    try:
        for step in steps:
            if step[0] == "attribute":
                if isinstance(message, dict):
                    message = message[step[1]]
                else:
                    message = getattr(message, step[1])
            elif step[0] == "indexing":
                indices = [int(s[1:]) for s in step[1].split("]")[:-1]]
                for index in indices:
                    if isinstance(message, List):
                        message = message[index]
                    else:
                        raise TypeError(
                            "%s object is not subscriptable" % type(message).__name__
                        )
    except (AttributeError, KeyError, TypeError) as e:
        if raise_error:
            raise e
        return None

    return message
